"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApiApp = void 0;
require("reflect-metadata");
const inversify_1 = require("inversify");
const Server_1 = require("./api/Server");
const AppConfig_1 = require("./model/AppConfig");
const LogFactory_1 = require("./util/logging/LogFactory");
/**
 * Application base class which combines the `Server`, `Container` (see `InversifyJS`)
 * and `AppConfig` classes to create a decorator driven API with typescript
 * middleware and dependency injection. It uses the `lambda-api` package as the
 * underlying HTTP API framework.
 *
 * AWS Lambda requests are handled by the `run` method, which will return a response
 * compatible with either API Gateway or an ALB.
 *
 * Extending this class will allow creating an app implementation for runtimes,
 * AWS Lambda, Local Web Server etc.
 */
class ApiApp {
    get middlewareRegistry() {
        return this.apiServer.middlewareRegistry;
    }
    /**
     * Create a new app.
     *
     * @param controllersPath (Optional) Paths to the directories that contain controller `js` files that
     *                        declare controllers. Required if the default `Container` is used, or the
     *                        provided `Container` instance has its `autoBindInjectable` flag set to `true`.
     *                        Ignored if the provided `Container` instance has its `autoBindInjectable`
     *                        flag set to `false`.
     * @param appConfig (Optional) Application config to pass to `lambda-api`, defaults to new `AppConfig`.
     * @param appContainer (Optional) `InversifyJS` IOC `Container` instance which can
     *                     build controllers and error interceptors, defaults to new `Container` with
     *                     `autoBindInjectable` flag set to `true`.
     */
    constructor(controllersPath, appConfig = new AppConfig_1.AppConfig(), appContainer = new inversify_1.Container({ autoBindInjectable: true })) {
        this.controllersPath = controllersPath;
        this.appConfig = appConfig;
        this.appContainer = appContainer;
        let autoInjectionEnabled = typeof (appContainer.options) === "object"
            && appContainer.options.autoBindInjectable === true;
        if (autoInjectionEnabled) {
            if (!Array.isArray(controllersPath) || controllersPath.length < 1) {
                throw new Error("controllersPath passed to ApiApp was not an array");
            }
            if (controllersPath.length < 1) {
                throw new Error("controllersPath passed to ApiApp was empty");
            }
            if (controllersPath.findIndex(p => typeof p !== "string" || p.trim() === "") > -1) {
                throw new Error("One or more paths in controllersPaths passed to ApiApp was null, empty or whitespace");
            }
        }
        appContainer.bind(AppConfig_1.AppConfig).toConstantValue(this.appConfig);
        this.apiServer = new Server_1.Server(appContainer, this.appConfig);
        this.logFactory = new LogFactory_1.LogFactory(appConfig);
        this.logger = this.logFactory.getLogger(ApiApp);
        this.initialised = false;
    }
    /**
     * Configure the `InversifyJS` IOC `Container` instance.
     *
     * @param configureBlock Function that takes a `Container` instance as a parameter.
     */
    configureApp(configureBlock) {
        configureBlock(this.appContainer);
    }
    /**
     * Configure the `API` instance from the `lambda-api` package.
     *
     * @param handler Function that takes an `API` instance as a parameter.
     */
    configureApi(configureBlock) {
        this.apiServer.configure(configureBlock);
    }
    /**
     * Initialise all controllers and endpoints declared using decorators.
     */
    async initialiseControllers() {
        if (this.initialised) {
            this.logger.debug("Ignoring call to initialiseControllers, app has already been initialised");
            return;
        }
        this.logger.debug("Initialising app controllers");
        try {
            await this.apiServer.discoverAndBuildRoutes(this.controllersPath);
            this.initialised = true;
        }
        catch (ex) {
            this.logger.fatal("Error initialising API app:\n%s", ex instanceof Error ? ex.stack : ex);
            throw ex;
        }
    }
}
exports.ApiApp = ApiApp;
//# sourceMappingURL=ApiApp.js.map