"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RequestBuilder = void 0;
const ApiRequest_1 = require("../model/ApiRequest");
/**
 * Builds `ApiRequest` instances using the builder
 * pattern. Used for testing.
 */
class RequestBuilder {
    /**
     * Start building a HTTP GET request.
     *
     * @param path URL path to request.
     */
    static get(path) {
        return new RequestBuilder("GET", path);
    }
    /**
     * Start building a HTTP POST request.
     *
     * @param path URL path to request.
     */
    static post(path) {
        return new RequestBuilder("POST", path);
    }
    /**
     * Start building a HTTP PUT request.
     *
     * @param path URL path to request.
     */
    static put(path) {
        return new RequestBuilder("PUT", path);
    }
    /**
     * Start building a HTTP PATCH request.
     *
     * @param path URL path to request.
     */
    static patch(path) {
        return new RequestBuilder("PATCH", path);
    }
    /**
     * Start building a HTTP DELETE request.
     *
     * @param path URL path to request.
     */
    static delete(path) {
        return new RequestBuilder("DELETE", path);
    }
    /**
     * Start building a HTTP request.
     *
     * @param method HTTP method to use.
     * @param path URL path to request.
     */
    static do(method, path) {
        return new RequestBuilder(method, path);
    }
    constructor(method, path) {
        this.method = method;
        this.path = path;
        this.httpHeaders = {};
        this.httpQueryParams = {};
        this.isBase64Encoded = false;
    }
    /**
     * Add a HTTP header to the request.
     *
     * @param name Name of the header.
     * @param value Value to set.
     * @returns This builder instance.
     */
    header(name, value) {
        this.httpHeaders[name] = value;
        return this;
    }
    /**
     * Add multiple HTTP headers to the request.
     *
     * @param httpHeaders Map of HTTP headers to add.
     * @returns This builder instance.
     */
    headers(httpHeaders) {
        for (let key in httpHeaders) {
            if (!httpHeaders.hasOwnProperty(key)) {
                continue;
            }
            this.httpHeaders[key] = httpHeaders[key];
        }
        return this;
    }
    /**
     * Add a HTTP query parameter to the request.
     *
     * @param name Name of the query param.
     * @param value Value to set.
     * @returns This builder instance.
     */
    query(key, value) {
        this.httpQueryParams[key] = value;
        return this;
    }
    /**
     * Add multiple HTTP query parameters to the request.
     *
     * @param params Map of HTTP query params to add.
     * @returns This builder instance.
     */
    queryParams(params) {
        for (let key in params) {
            if (!params.hasOwnProperty(key)) {
                continue;
            }
            this.httpQueryParams[key] = params[key];
        }
        return this;
    }
    /**
     * Add a HTTP body to the request.
     *
     * @param value The body as a raw string.
     * @returns This builder instance.
     */
    body(value) {
        this.httpBody = value;
        return this;
    }
    /**
     * Add a base64 encoded HTTP body to the request.
     *
     * @param value The body as a base64 string.
     * @returns This builder instance.
     */
    base64EncodedBody(value) {
        this.body(value);
        this.isBase64Encoded = true;
        return this;
    }
    /**
     * Add a binary HTTP body to the request.
     *
     * @param value The binary body as a Buffer.
     * @returns This builder instance.
     */
    binaryBody(value) {
        this.base64EncodedBody(value.toString("base64"));
        return this;
    }
    /**
     * Add basic authentication to the request.
     *
     * @param username Username to send.
     * @param password Password to send.
     * @returns This builder instance.
     */
    basicAuth(username, password) {
        let credentials = Buffer.from(`${username}:${password}`).toString("base64");
        this.httpHeaders.Authorization = `Basic ${credentials}`;
        return this;
    }
    mapToObject(map) {
        let obj = {};
        for (let key in map) {
            if (!map.hasOwnProperty(key)) {
                continue;
            }
            obj[key] = map[key];
        }
        return obj;
    }
    /**
     * Build a request object using the current builder config.
     */
    build() {
        let request = new ApiRequest_1.ApiRequest();
        request.path = this.path;
        request.httpMethod = this.method;
        request.headers = this.mapToObject(this.httpHeaders);
        request.queryStringParameters = this.mapToObject(this.httpQueryParams);
        request.body = this.httpBody;
        request.isBase64Encoded = this.isBase64Encoded;
        return request;
    }
}
exports.RequestBuilder = RequestBuilder;
//# sourceMappingURL=RequestBuilder.js.map